@@ -12,7 +12,7 @@ import logging
 import copy
 
 import pyomo.util.plugin
-from pyomo.opt import SolverFactory
+from pyomo.opt import SolverFactory, PersistentSolver
 from pyomo.core import *
 from pyomo.pysp import phextension
 from pyomo.pysp.plugins.phboundextension import (_PHBoundBase,
@@ -109,13 +109,7 @@ class convexhullboundextension(pyomo.util.plugin.SingletonPlugin, _PHBoundBase):
               ("inner" if self._is_minimizing else "outer"))
 
         # push the updated outer bound to PH, for reporting purposes.
-        if ph._reported_outer_bound is None:
-            ph._reported_outer_bound = self._outer_bound_history[storage_key]
-        else:
-            if self._is_minimizing:
-                ph._reported_outer_bound = max(self._outer_bound_history[storage_key], ph._reported_outer_bound)
-            else:
-                ph._reported_outer_bound = min(self._outer_bound_history[storage_key], ph._reported_outer_bound)
+        ph._update_reported_bounds(self._outer_bound_history[storage_key])
 
         # Deactivate the weight terms.
         self.DeactivatePHObjectiveWeightTerms(ph)
@@ -152,14 +146,7 @@ class convexhullboundextension(pyomo.util.plugin.SingletonPlugin, _PHBoundBase):
                     self.ComputeInnerBound(ph, storage_key)
 
         # push the updated inner bound to PH, for reporting purposes.
-        if ph._reported_inner_bound == None:
-            ph._reported_inner_bound = self._inner_bound_history[storage_key]
-        else:
-            if self._is_minimizing:
-                ph._reported_inner_bound = min(self._inner_bound_history[storage_key], ph._reported_inner_bound)
-            else:
-                ph._reported_inner_bound = max(self._inner_bound_history[storage_key], ph._reported_inner_bound)
-
+        ph._update_reported_bounds(inner = self._inner_bound_history[storage_key])
 
         # Restore ph to its state prior to entering this method (e.g.,
         # fixed variables, scenario solutions, proximal terms,
@@ -277,17 +264,21 @@ class convexhullboundextension(pyomo.util.plugin.SingletonPlugin, _PHBoundBase):
 
 #        print "V_BOUNDS CONSTRAINT:"
 #        self._master_model.V_Bound.pprint()
-
-
-        solver = SolverFactory("cplex")
-        results=solver.solve(self._master_model,tee=False,load_solutions=False)
-        self._master_model.solutions.load_from(results)
+        with SolverFactory(ph._solver_type,
+                           solver_io=ph._solver_io) as solver:
+            # the reason we go through this trouble rather
+            # than harcoding cplex as the solver is so that
+            # we can test the script and not have to worry
+            # about a missing solver
+            if isinstance(solver, PersistentSolver):
+                solver.compile_instance(self._master_model)
+            results = solver.solve(self._master_model)
+            self._master_model.solutions.load_from(results)
 #        print "MASTER MODEL WVAR FOLLOWING SOLVE:"
 #        self._master_model.pprint()
 
 #        self._master_model.pprint()
 
-
     #
     # take the weights from the current convex hull master problem
     # solution, and push them into the PH scenario instances - so we
@@ -394,6 +385,7 @@ class convexhullboundextension(pyomo.util.plugin.SingletonPlugin, _PHBoundBase):
         self._outer_bound_history[ph_iter], \
             self._outer_status_history[ph_iter] = \
                 self.ComputeOuterBound(ph, ph_iter)
+        ph._update_reported_bounds(outer = self._outer_bound_history[ph_iter]) # dlw May 2016
 
         # YIKES - WHY IS THIS HERE????!!
         self._populate_bundle_dual_master_model(ph)
